#!/bin/bash
# Copyright (C) 2009-2012 Centro de Computacao Cientifica e Software Livre
# Departamento de Informatica - Universidade Federal do Parana - C3SL/UFPR
#
# This file is part of buildpackage
#
# buildpackage is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
# USA.

# This script compiles the agent files (collect and client) and prepares the
# directories tree necessary to build the installation and update packages. Exit
# status:
#
# 0 - Success
# 1 - Error while trying to load conf file
# 2 - Error: wget wasn't compiled
# 3 - Error: dynamically compiled wget
# 4 - Error while executing git pull
# 5 - Error while trying to compile the client
# 6 - Error while trying to compile the collect-parser

# ------------------------------------------------------------------------------
PREFIX="$(dirname $0)"
GITBASEDIR="../"
WEBSERVICE="seed.c3sl.ufpr.br"

# Get the configuration
if ! source "${PREFIX}/buildpackage.conf" 2> /dev/null; then
    printf "ERROR (1): Failed to load buildpackage.conf.\n"
    exit 1
fi

# Customizable variables. Please refer to buildpackage.conf to change these
# values
: ${AGENTDIR:="agent-files"}
: ${AGENTPACKAGEDIR:="SEED2-run"}

printf "====================\n$0 v0.1\n\n"
printf "**** Make sure that ${WEBSERVICE} webservice are up and running!!!\n"
printf "\nNOTE: This will take a while.\n"
printf "      Make sure the git update process below will run ok.\n"
printf "====================\n\n"

# ------------------------------------------------------------------------------
# Check if wget was built
if ! test -f "${PREFIX}/wget-"*"/src/wget"; then
    printf "ERROR (2): Please run build-wget before running this script. %b" \
           "Aborting...\n"
    exit 2
fi

# Check if wget is a static binary
ldd "${PREFIX}/wget-"*"/src/wget" &> /dev/null
if ! test $? -eq 1; then
    printf "ERROR (3): The wget binary wasn't compiled with static library.%b" \
           "\nPlease, build wget using build-wget. Aborting...\n"
    exit 3
fi

# Ensure $GITBASEDIR directory exists
mkdir -p "${GITBASEDIR}"

# Ensure the repository is updated
printf "Updating git repository...\n"
if ! git pull; then
    printf "ERROR (4): Something went wrong.\n Problems executing %b" \
           "git pull(1). Aborting...\n"
    exit 4
fi
printf "\n\n"

# ------------------------------------------------------------------------------
# Compile the collect agent and the client
printf "Starting compilation process...\n\n"
printf "Starting client compilation...\n\n"
sleep 1

cd "${GITBASEDIR}/client"
if make && make install; then
    printf "\n\nFinished client compilation.\n%b" \
           "Everything was fine.\n\n"
else
    printf "\n\n"
    printf "ERROR (5): Finished client compilation.\n%b" \
           "Something went wrong. Aborting...\n\n"
    exit 5
fi
cd -

printf "Starting collect-parser compilation...\n\n"
sleep 1

cd "${GITBASEDIR}/collect"
if make && make install; then
    printf "\n\n"
    printf "Finished collect-parser compilation.\n%b" \
           "Everything was fine.\n\n"
else
    printf "\n\n"
    printf "ERROR (6): Finished collect-parser compilation.\n%b" \
           "Something went wrong. Aborting...\n\n"
    exit 6
fi
cd -

# Build the package itself
printf "Starting to build the package...\n"
OUTDIR="${AGENTDIR}/${AGENTPACKAGEDIR}/"

rm -Rf "${OUTDIR}"
mkdir -p "${OUTDIR}/client/"
mkdir -p "${OUTDIR}/lib/"

EXCLUDEFILE=$(tempfile --prefix=build-package-exclude)
cat > ${EXCLUDEFILE} << EOF
src
lib/include
lib/libxml2*/*
lib/*.tar.gz
Makefile
COPYING
INSTALL
NEWS
README
AUTHORS
TODO
tests
version
net/*.c
EOF

printf "Copying collect files... "
rsync -aHx --delete --exclude-from=${EXCLUDEFILE} \
      "${GITBASEDIR}/collect/"* "${OUTDIR}"
rm -Rf "${OUTDIR}/lib/libxml2"*
printf "done.\n"

cat > ${EXCLUDEFILE} << EOF
src
lib
Makefile
run-config.sh
COPYING
INSTALL
NEWS
README
AUTHORS
TODO
tests
EOF

printf "Copying client files... "
rsync -aHx --delete --exclude-from=${EXCLUDEFILE} \
       "${GITBASEDIR}/client/"* "${OUTDIR}/client/"
printf "done.\n"

BOOTSTRAP="${PREFIX}/bootstraps/bootstrap-install"
printf "Copying bootstrap... "
cp -a "${BOOTSTRAP}" "${AGENTDIR}"/
printf "done.\n"

printf "Copying wget binary... "
cp -a "${PREFIX}/wget-"*"/src/wget" "${OUTDIR}/bin/"
printf "done.\n"

rm -f "${EXCLUDEFILE}"

printf "\nAll done.\n\n"
