#!/bin/bash
# Copyright (C) 2009-2012 Centro de Computacao Cientifica e Software Livre
# Departamento de Informatica - Universidade Federal do Parana - C3SL/UFPR
#
# This file is part of buildpackage
#
# buildpackage is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
# USA.

# This script builds the installation package. Exit status:
#
# 0 - Success
# 1 - Error: empty INEP
# 2 - Error: some files are corrupt
# 3 - Error while trying to copy the agent files
# 4 - Error while trying to copy the installation bootstrap
# 5 - Error while trying to compress the agent files

# ------------------------------------------------------------------------------
# This function checks the integrity of one file
check_file_integrity(){
    FILE="$*"
    FILEREAL="$(echo ${FILE} | sed "s#^${MD5DIRESC}##g")"

    test -f "${AGENT_FILES}/SEED2-run/${FILEREAL}" || return 1
    (md5sum "${AGENT_FILES}/SEED2-run/${FILEREAL}" | cut -d' ' -f1 | \
     diff - "${FILE}" &> /dev/null) || return 1
}

AGENT_FILES="./agent-files"
BOOTSTRAP="bootstrap-install"
INEP=$1
MD5DIR="${AGENT_FILES}/SEED2-run/.md5sum"
MD5DIRESC="$(echo ${MD5DIR} | sed "s/\./\\\./g")"

# Ensure proxy variables are empty
PHOST=
PPORT=
PUID=
PPASSWD=

# If the parameter isn't quotation marks, add its value to the respective proxy
# information variable
test "$2" = '""' || PHOST=$2
test "$3" = '""' || PPORT=$3
test "$4" = '""' || PUID=$4
test "$5" = '""' || PPASSWD=$5

# Check if $INEP is empty
if test -z "${INEP}"; then
    printf "ERROR (1): Empty INEP.\n"
    exit 1
fi

# -----------------------------------------------------------------------------
# Check all MD5 files
printf "Checking SEED2-run files integrity... "

for FILE in $(find "${MD5DIR}" -type f); do
    if ! check_file_integrity "${FILE}"; then
        printf "ERROR (2): File(s) corrupted. Please, re-create integrity or %b"
               "rebuild the agent files.\n"
        exit 2
    fi
done

printf "done.\n"

# -----------------------------------------------------------------------------
# Start the tree creation process, removing any possible garbage before
# creating the package specific temporary directory (using the INEP)
printf "Creating tree... "

rm -rf "/tmp/${INEP}"
mkdir -p "/tmp/${INEP}"

# Copy the agent files to the temporary directory where the installation package
# will be generated
if ! cp -r "${AGENT_FILES}/SEED2-run" "/tmp/${INEP}/SEED2"; then
    printf "ERROR (3): Failed to copy agent files.\n"
    exit 3
fi

# Create client conf directory and the inep file with the school INEP
mkdir -p "/tmp/${INEP}/SEED2/client/conf"
echo "${INEP}" > "/tmp/${INEP}/SEED2/client/conf/inep"

# Create the md5 files of inep and inep backup
FILE="/tmp/${INEP}/SEED2/client/conf/inep"
md5sum "${FILE}" |cut -d' ' -f1 > \
    "/tmp/${INEP}/SEED2/.md5sum/client/conf/inep"
cp -a "${FILE}" "/tmp/${INEP}/SEED2/.backup/client/conf/inep"
FILE="/tmp/${INEP}/SEED2/.backup/client/conf/inep"
md5sum "${FILE}" |cut -d' ' -f1 > \
    "/tmp/${INEP}/SEED2/.md5sum/.backup/client/conf/inep"

# Create the proxy file with all necessary proxy information, and its md5
echo "phost=$PHOST" > "/tmp/${INEP}/SEED2/client/conf/proxy"
echo "pport=$PPORT" >> "/tmp/${INEP}/SEED2/client/conf/proxy"
echo "puid=$PUID" >> "/tmp/${INEP}/SEED2/client/conf/proxy"
echo "ppasswd=$PPASSWD" >> "/tmp/${INEP}/SEED2/client/conf/proxy"
FILE="/tmp/${INEP}/SEED2/client/conf/proxy"
md5sum "${FILE}" |cut -d' ' -f1 > \
    "/tmp/${INEP}/SEED2/.md5sum/client/conf/proxy"

# Create the backup proxy file and its md5
cp -a "${FILE}" "/tmp/${INEP}/SEED2/.backup/client/conf/proxy"
FILE="/tmp/${INEP}/SEED2/.backup/client/conf/proxy"
md5sum "${FILE}" |cut -d' ' -f1 > \
    "/tmp/${INEP}/SEED2/.md5sum/.backup/client/conf/proxy"

# Copy the bootstrap-install to the temporary directory
if ! cp -r "${AGENT_FILES}/${BOOTSTRAP}" "/tmp/${INEP}"; then
    printf "ERROR (4): Failed to copy ${BOOTSTRAP} to the %b" \
           "temporary directory.\n"
    exit 4
fi

printf "done.\n"
# End of tree creation process

# -----------------------------------------------------------------------------
# The tree is ready, we need to compress it now (tar+gz)
printf "Packaging... "

cd "/tmp/${INEP}"
if ! tar -czf SEED2.tgz SEED2; then
    printf "ERROR (5): Failed to compress the agent package.\n"
    exit 5
fi

# Create the tar md5 and compute the bootstrap lines
MD5_CHECK=$(md5sum SEED2.tgz | awk '{print $1}')
LINENUM=$(( $(wc -l "${BOOTSTRAP}" | cut -d' ' -f1) + 1 ))

# Edit the bootstrap and start the real package
sed "s/BOOTSTRAP_LINES=/BOOTSTRAP_LINES=${LINENUM}/g; \
     s/MD5_CHECK=/MD5_CHECK=\"${MD5_CHECK}\"/g" \
    "${BOOTSTRAP}" > agent_${INEP}.run

# Insert the tar on the package and finish
cat "SEED2.tgz" >> "agent_${INEP}.run"
mv "agent_${INEP}.run" "/tmp"

printf "done.\n"
# End of packaging step

# -----------------------------------------------------------------------------
# Remove temporary files
printf "Removing temporary files... "

cd ..
rm -rf "${INEP}" 2> /dev/null

printf "done.\n"
# End of temporary files removal

printf "\nAll done.\n"
