#!/bin/bash
# Copyright (C) 2009-2012 Centro de Computacao Cientifica e Software Livre
# Departamento de Informatica - Universidade Federal do Parana - C3SL/UFPR
#
# This file is part of buildpackage
#
# buildpackage is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
# USA.

# This script builds the update package. Exit status:
#
# 0 - Success
# 1 - Error while trying to load conf file
# 2 - Error: version file is not readable
# 3 - Error: some files are corrupt
# 4 - Error while trying to compress the agent files
# 5 - Error: MD5 directory is missing

# ------------------------------------------------------------------------------
# This function checks the integrity of one file
check_file_integrity(){
    FILE="$*"
    FILEREAL="$(echo ${FILE} | sed "s#^${MD5DIRESC}##g")"

    test -f "${AGENTPACKAGEDIR}/${FILEREAL}" || return 1
    (md5sum "${AGENTPACKAGEDIR}/${FILEREAL}" | cut -d' ' -f1 | \
     diff - "${FILE}" &> /dev/null) || return 1
}

# ------------------------------------------------------------------------------
PREFIX="$(dirname $0)"
BOOTSTRAP="${PREFIX}/bootstraps/bootstrap-update"

# Get configuration
if ! source "${PREFIX}/buildpackage.conf" 2> /dev/null; then
    printf "ERROR (1): Failed to load buildpackage.conf.\n"
    exit 1
fi

# Customizable variables. Please refer to buildpackage.conf to change these
# values
: ${AGENTDIR:="agent-files"}
: ${AGENTPACKAGEDIR:="SEED2-run"}
: ${UPDATEDIR:="update-files"}

MD5DIR="${AGENTPACKAGEDIR}/.md5sum"
MD5DIRESC="$(echo ${MD5DIR} | sed "s/\./\\\./g")"
VERSIONFILE="${AGENTDIR}/${AGENTPACKAGEDIR}/client/conf/version"

# ------------------------------------------------------------------------------
printf "====================\n"
printf "$0 v0.1\n\n"
printf "*** Make sure ${AGENTDIR}/${AGENTPACKAGEDIR}\n"
printf "contains the latest package generated by build-agent-files ***\n"
printf "====================\n\nStarting to build the package...\n\n"

# Check the readability of version file and get the version
if ! test -r "${VERSIONFILE}"; then
    printf "ERROR (2): The version file is not readable. Aborting...\n"
    exit 2
fi
VERSION=$(cat ${VERSIONFILE})

# Set the output file name
OUTPUT="${UPDATEDIR}/SEED2-update-${VERSION}.run"

# Change to agent directory
cd "${AGENTDIR}"

# ------------------------------------------------------------------------------
# Check all MD5 files
printf "Checking SEED2-run files integrity... "

if ! test -d "${MD5DIR}"; then
    printf "ERROR (5): There is no MD5 directory. Please, run create %b" \
           "integrity.\n"
    exit 5
fi

for FILE in $(find "${MD5DIR}" -type f); do
    if ! check_file_integrity "${FILE}"; then
        printf "ERROR (3): File(s) corrupted. Please, re-create integrity %b" \
               "or rebuild the agent files.\n"
        exit 3
    fi
done

printf "done.\n"

# ------------------------------------------------------------------------------
# Compress the agent files
printf "Compressing SEED2-run files... "

TARFILE=$(tempfile --prefix=SEED2-tar)
if ! tar -czf "${TARFILE}" SEED2-run; then
    printf "ERROR (4): Failed to compress the agent files.\n"
    exit 4
fi
cd - &> /dev/null

printf "done.\n"
# End of agent compression

# ------------------------------------------------------------------------------
# Calculate the MD5
printf "Calculating MD5... "

MD5_CHECK=$(md5sum ${TARFILE} | awk '{print $1}')

printf "done.\n"
# End of MD5 calculation

# ------------------------------------------------------------------------------
# Generate the update package
printf "Creating update .run package... "

mkdir -p "${UPDATEDIR}"
LINENUM=$(( $(wc -l "${BOOTSTRAP}" | cut -d' ' -f1) + 1 ))

sed "s/BOOTSTRAP_LINES=/BOOTSTRAP_LINES=${LINENUM}/g; \
     s/FILE=/FILE=\"SEED2-update-${VERSION}.tgz\"/g; \
     s/MD5_CHECK=/MD5_CHECK=\"${MD5_CHECK}\"/g" \
     "${BOOTSTRAP}" > "${OUTPUT}"

cat "${TARFILE}" >> "${OUTPUT}"

printf "done.\n"
# End of update package generation

# Remove the tar file
rm -f "${TARFILE}"

printf "\nAll done.\n"
