#!/bin/bash
# Copyright (C) 2009-2012 Centro de Computacao Cientifica e Software Livre
# Departamento de Informatica - Universidade Federal do Parana - C3SL/UFPR
#
# This file is part of collect-agent
#
# collect-agent is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
# USA.

# This script checks the agente files' integrity and restore them from backup if
# needed. In case the backup doesn't work, an update is forced to occur. It also
# verifies if some update went wrong and tries to redo it. Exit status:
#
# 0 - Success
# 1 - Error: could not restore files nor force update

PREFIX="$(dirname $(readlink -f $0))"
PREFIXESC="$(echo ${PREFIX} | sed "s/\./\\\./g")"

MD5DIR="${PREFIX}/.md5sum"
MD5DIRESC="$(echo ${MD5DIR} | sed "s/\./\\\./g")"

BACKUPNAME=".backup"
BACKUPDIR="${PREFIX}/${BACKUPNAME}"

LOGFILE=${PREFIX}/log/collect.log

# ------------------------------------------------------------------------------
# Try to restore corrupted file
restore_backup(){
    # If the corrupted file is a backup file
    # will try to restore from the original file
    if echo "${FILEREAL}" |grep -q "${BACKUPNAME}"; then
        FILE="$(echo ${FILE} |sed "s#${BACKUPNAME}##g")"
        FILEREAL="$(echo ${FILEREAL} |sed "s#${BACKUPNAME}##g")"

        echo "$FILE $FILEREAL"
        (md5sum "${PREFIX}/${FILEREAL}" |cut -d' ' -f1 | \
         diff - "${FILE}" &> /dev/null) || return 1
        mkdir -p "$(dirname "${BACKUPDIR}/${FILEREAL}")"
        cp -a "${PREFIX}/${FILEREAL}" "${BACKUPDIR}/${FILEREAL}"
        return $?
    fi

    # This is a original file, so will check if there is some backup for it
    if test -e "${BACKUPDIR}/${FILEREAL}"; then
        (md5sum "${BACKUPDIR}/${FILEREAL}" |cut -d' ' -f1 | \
         diff - "${MD5DIR}/.backup/${FILEREAL}" &> /dev/null) || return 1
        cp -a "${BACKUPDIR}/${FILEREAL}" "${PREFIX}/${FILEREAL}"
        return $?
    fi

    return 1
}

# ------------------------------------------------------------------------------
# Function that checks the integrity of one file. If the file is corrupted, try
# to restore it from a backup
check_file_integrity(){
    FILE="$*"
    FILEREAL="$(echo ${FILE} |sed "s#^${MD5DIRESC}##g")"

    test -f "${PREFIX}/${FILEREAL}" || restore_backup || return 1
    (md5sum "${PREFIX}/${FILEREAL}" |cut -d' ' -f1 | \
     diff - "${FILE}" &> /dev/null) || restore_backup || return 1
}

# ------------------------------------------------------------------------------
# Function that checks if all necessary files to update the agent are ok. If the
# function is not able to restore some file, this update can't happen
try_force_update(){
    DEPENDENTFILES="update.sh agent.conf load-config.sh client/run.sh
                    client/common.sh client/bin/client client/conf/version"

    for FILE in ${DEPENDENTFILES}; do
        check_file_integrity "${MD5DIR}/${FILE}" || return 1
    done

    # Make sure the agent will update itself
    echo "0.0.0" > "${PREFIX}/client/conf/version"

    # All needed files are ok, so call the update script
    source "${PREFIX}/update.sh" || return 1

    return 0
}

# ------------------------------------------------------------------------------
# Function that tries to autoupdate in a simpler way
# If everything goes wrong, this is our last hope
update_myself(){
    UPDATETRIES=3
    UPDATETIMEOUT=100
    UPDATESERVER="http://200.17.202.187/pacotes/download"
    TMPDIR="${PREFIX}/tmp"

    # The proxy config itself
    if ! check_file_integrity "${MD5DIR}/client/conf/proxy"; then
        PROXY=""
        puid=""
        ppasswd=""
    else
        PROXYCONF="${PREFIX}/client/conf/proxy"
        phost=$(grep "^phost=" ${PROXYCONF} | cut -f2 -d=)
        pport=$(grep "^pport=" ${PROXYCONF} | cut -f2 -d=)
        puid=$(grep "^puid=" ${PROXYCONF} | cut -f2 -d=)
        ppasswd=$(grep "^ppasswd=" ${PROXYCONF} | cut -f2 -d=)

        if test -z "$phost"; then
            PROXY=""
        else
            PROXY="http://${phost}:${pport}"
        fi
    fi

    PROXYPASSWD="--proxy-password"
    if which wget &> /dev/null; then
        WGETBIN="$(which wget)"
        WGETVERSION="$(${WGETBIN} --version | head -n1 | cut -d' ' -f3)"
        WGETVER1="$(echo ${WGETVERSION} | cut -d'.' -f1)"
        WGETVER2="$(echo ${WGETVERSION} | cut -d'.' -f2)"
        # If the system wget(1) is older than the 1.10 version, use
        # the option "--proxy-passwd" to pass the proxy password to wget(1)
        if test ${WGETVER1} -lt 1 -o ${WGETVER1} -eq 1 -a ${WGETVER2} -lt 10
        then
            PROXYPASSWD="--proxy-passwd"
        fi
    else
        WGETBIN="${PREFIX}/bin/wget"
    fi

    http_proxy="${PROXY}" ${WGETBIN} --tries=${UPDATETRIES} \
                                     --timeout=${UPDATETIMEOUT} \
                                     --proxy-user="${puid}" \
                                     ${PROXYPASSWD}="${ppasswd}" \
                                     "${UPDATESERVER}/SEED2-update.run" \
                                     -O "${TMPDIR}/SEED2-update.run" \
                                     &> /dev/null || return 1

    # The package was downloaded, now try to update the agent
    bash "${TMPDIR}/SEED2-update.run" || return 1
    rm -f "${TMPDIR}/SEED2-update.run" &> /dev/null

    return 0
}

# ------------------------------------------------------------------------------
exec >> ${LOGFILE} 2>&1

date +"%F %T - Start of integrity checker execution."

# First, check if some update went wrong
if test -e "${PREFIX}/updating" || ! test -d "${MD5DIR}"; then
    try_force_update || update_myself ||
        (printf "ERROR: Forced update didn't work.\n" && exit 1)
    rm -f "${PREFIX}/updating" 2> /dev/null
else
    # Check all MD5 files
    for FILE in $(find "${MD5DIR}" -type f); do
        check_file_integrity ${FILE} ||
            ((try_force_update || update_myself) && exit 0) ||
                (printf \
                    "ERROR: Could not restore files nor update the agent.\n" &&
                    exit 1)
    done
fi

date +"%F %T - Integrity checker terminated successfully."
exit 0
