/*
 * Copyright (C) 2009-2012 Centro de Computacao Cientifica e Software Livre
 * Departamento de Informatica - Universidade Federal do Parana - C3SL/UFPR
 *
 * This file is part of server
 *
 * collect-agent is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 * USA.
 */

/* General libs */
import java.util.*;
import java.text.*;

/* Database libs */
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.Statement;
import java.sql.SQLException;
import java.sql.Timestamp;
import org.postgresql.Driver;

/* File libs */
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;

/* XML Parser libs */
import org.jdom.*;
import java.io.StringReader;
import org.xml.sax.InputSource;

/* DOM */
import org.jdom.input.DOMBuilder;
import org.apache.xerces.parsers.DOMParser;
import org.w3c.dom.*;

/* Regular Expressions */
import java.util.regex.Pattern;
import java.util.regex.Matcher;

public class ServerTesting {

/**
 * Return a string with database url.
 *
 * @author              Danilo Kiyoshi Simizu Yorinori
 * @param     host      DB Host
 * @param     port      DB Port
 * @param     schema    DB Schema
 * @return              String
 */
private static String createDbConnectionString(String host, String port,
                                               String schema) {
    StringBuilder sb = new StringBuilder("jdbc:postgresql://");
    sb.append(host);
    sb.append(":");
    sb.append(port);
    sb.append("/");
    sb.append(schema);

    return sb.toString();
}

/**
 * Return a string with availability insertion query.
 *
 * @author                  Danilo Kiyoshi Simizu Yorinori
 * @param     table         DB Table
 * @param     inep          School's INEP number
 * @param     mac           Host MAC-Address
 * @param     timestamp     Timestamp
 * @return                  String
 */
private static String createAvailabilityQuery(String table, String inep,
                                              String mac, String timestamp) {
    StringBuilder sb = new StringBuilder("INSERT INTO ");
    sb.append(table);
    sb.append(" (sa_inep, sa_mac, sa_data) VALUES (");
    /* Inep */
    if (inep != null) {
        sb.append("'");
        sb.append(inep);
        sb.append("', ");
    } else {
        sb.append("null, ");
    }
    /* MAC Address */
    if (mac != null) {
        sb.append("'");
        sb.append(mac);
        sb.append("', ");
    } else {
        sb.append("null, ");
    }
    /* Timestamp */
    if (timestamp != null) {
        sb.append("'");
        sb.append(timestamp);
        sb.append("')");
    } else {
        sb.append("null)");
    }

    return sb.toString();
}

/**
 * Return a string with id verification query.
 *
 * @author            Danilo Kiyoshi Simizu Yorinori
 * @param     table   DB Table
 * @param     inep    School's INEP number
 * @return            String
 */
private static String createIdVerificationQuery(String table, String inep) {
    StringBuilder sb = new StringBuilder("SELECT * FROM ");
    sb.append(table);
    sb.append(" WHERE esc_inep=");
    /* Inep */
    if (inep != null) {
        sb.append("'");
        sb.append(inep);
        sb.append("', ");
    } else {
        sb.append("null");
    }

    return sb.toString();
}

/**
 * Return a string with inventory insertion query.
 *
 * @author                  Danilo Kiyoshi Simizu Yorinori
 * @param     table         DB Table
 * @param     inep          School's INEP number
 * @param     mac           Host MAC-Address
 * @param     hdd1Model     Hdd 1 Model
 * @param     hdd1Size      Hdd 1 Size (GB)
 * @param     hdd1Used      Hdd 1 Used
 * @param     hdd2Model     Hdd 2 Model
 * @param     hdd2Size      Hdd 2 Size (GB)
 * @param     hdd2Used      Hdd 2 Used
 * @param     hddAd         Number of additional Hdd's
 * @param     memSize       Total Memory size (MB)
 * @param     model         Processor model
 * @param     type          OS type
 * @param     distro        OS distro
 * @param     kernel        OS kernel
 * @param     useUp         Network Upload use
 * @param     useDown       Network Download use
 * @param     timestamp     Timestamp
 * @return                  String
 */
private static String createInventoryQuery(String table, String inep,
        String mac, String hdd1Model, String hdd1Size, String hdd1Used,
        String hdd2Model, String hdd2Size, String hdd2Used, String hddAd,
        String memSize, String model, String type, String distro,
        String kernel, String useUp, String useDown, String timestamp) {
    StringBuilder sb = new StringBuilder("INSERT INTO ");
    sb.append(table);
    sb.append(" (sa_inep, sa_mac, sa_disco1_modelo, sa_disco1_capacidade,");
    sb.append(" sa_disco1_usado, sa_disco2_modelo, sa_disco2_capacidade,");
    sb.append(" sa_disco2_usado, sa_discos_ad, sa_memoria, sa_processador,");
    sb.append(" sa_so_nome, sa_so_distribuicao, sa_so_kernel, sa_uso_up,");
    sb.append(" sa_uso_down, sa_data) VALUES (");

    /* Inep */
    if (inep != null) {
        sb.append("'");
        sb.append(inep);
        sb.append("', ");
    } else {
        sb.append("null, ");
    }
    /* MAC Address */
    if (mac != null) {
        sb.append("'");
        sb.append(mac);
        sb.append("', ");
    } else {
        sb.append("null, ");
    }
    /* Hdd 1 Model */
    if (hdd1Model != null) {
        sb.append("'");
        sb.append(hdd1Model);
        sb.append("', ");
    } else {
        sb.append("null, ");
    }
    /* Hdd 1 Size */
    if (hdd1Size != null) {
        sb.append(hdd1Size);
        sb.append(", ");
    } else {
        sb.append("null, ");
    }
    /* Hdd 1 Used */
    if (hdd1Used != null) {
        sb.append(hdd1Used);
        sb.append(", ");
    } else {
        sb.append("null, ");
    }
    /* Hdd 2 Model */
    if (hdd2Model != null) {
        sb.append("'");
        sb.append(hdd2Model);
        sb.append("', ");
    } else {
        sb.append("null, ");
    }
    /* Hdd 2 Size */
    if (hdd2Size != null) {
        sb.append(hdd2Size);
        sb.append(", ");
    } else {
        sb.append("null, ");
    }
    /* Hdd 2 Used */
    if (hdd2Used != null) {
        sb.append(hdd2Used);
        sb.append(", ");
    } else {
        sb.append("null, ");
    }
    /* Hdd Ad */
    if (hddAd != null) {
        sb.append(hddAd);
        sb.append(", ");
    } else {
        sb.append("null, ");
    }
    /* Memory Size */
    if (memSize != null) {
        sb.append(memSize);
        sb.append(", ");
    } else {
        sb.append("null, ");
    }
    /* Processor Model */
    if (model != null) {
        sb.append("'");
        sb.append(model);
        sb.append("', ");
    } else {
        sb.append("null, ");
    }
    /* SO Type */
    if (type != null) {
        sb.append("'");
        sb.append(type);
        sb.append("', ");
    } else {
        sb.append("null, ");
    }
    /* SO Distro */
    if (distro != null) {
        sb.append("'");
        sb.append(distro);
        sb.append("', ");
    } else {
        sb.append("null, ");
    }
    /* SO Kernel */
    if (kernel != null) {
        sb.append("'");
        sb.append(kernel);
        sb.append("', ");
    } else {
        sb.append("null, ");
    }
    /* Use Up */
    if (useUp != null) {
        sb.append(useUp);
        sb.append(", ");
    } else {
        sb.append("null, ");
    }
    /* Use Down */
    if (useDown != null) {
        sb.append(useDown);
        sb.append(", ");
    } else {
        sb.append("null, ");
    }
    /* Timestamp */
    if (timestamp != null) {
        sb.append("'");
        sb.append(timestamp);
        sb.append("')");
    } else {
        sb.append("null)");
    }

    return sb.toString();
}

/**
 * Return a NodeList with child nodes of first child from other list of nodes.
 *
 * @author                  Danilo Kiyoshi Simizu Yorinori
 * @param     list          NodeList
 * @return                  NodeList
 */
private static NodeList getChildNodes(NodeList list) {
//XXX
    if (list.getLength() == 0) return null;
//XXX
    Node tag = list.item(0);
    if (tag.hasChildNodes()) {
        return tag.getChildNodes();
    } else {
        return null;
    }
}

/**
 * Verify the tag name if it is equal to a string passed by parameter.
 *
 * @author                  Danilo Kiyoshi Simizu Yorinori
 * @param     tag           Node with an XML tag
 * @param     name          String with tag name
 * @return                  Boolean
 */
private static Boolean verifyTagName(Node tag, String name) {
    String tagName = tag.getNodeName();
    if (tagName.equals(name)) {
        return true;
    } else {
        return false;
    }
}

/**
 * Return a string with value of attribute.
 *
 * @author                  Danilo Kiyoshi Simizu Yorinori
 * @param     tag           Node with an XML tag
 * @param     attr          Name of attribute
 * @return                  String
 */
private static String getAttributeValue(Node tag, String attr) {
    if (tag.hasAttributes()) {
        NamedNodeMap attrs = tag.getAttributes();
        Node nodeValue = attrs.getNamedItem(attr);
        if (nodeValue != null) {
            return nodeValue.getNodeValue();
        } else {
            return null;
        }
    } else {
        return null;
    }
}

/**
 * Return a string with timestamp generate by server time.
 *
 * @author                  Danilo Kiyoshi Simizu Yorinori
 * @return                  String
 */
private static String getTimestamp() {
    /* Get server time with timestamp format */
    Date date = new Date();
    long dateMilisecs = date.getTime();
    Timestamp dateTimestamp = new Timestamp(dateMilisecs);

    return dateTimestamp.toString();
}

/**
 * Return true if regular expression matches with string or false if does not.
 * The regular expression is pre-configured in Pattern object.
 *
 * @author                  Danilo Kiyoshi Simizu Yorinori
 * @param       pattern     Pattern object
 * @param       match       String that will be checked
 * @return                  Boolean
 */
private static Boolean matchRegExp (Pattern pattern, String match) {
    Matcher matcher = pattern.matcher(match);
    while(matcher.find()) {
        return true;
    }
    return false;
}

/**
 * Return a string with version, string with some error or null.
 * TODO: the return with null must be changed.
 *
 * @author                  Danilo Kiyoshi Simizu Yorinori
 * @return                  String
 */
private static String getVersionFromFile()
throws Exception {
    try {
        /* Version file path */
        String versionFile = "/home/webservice/apache-tomcat-6.0.20/webapps/axis/version";
        //String versionFile = "";

        /* Verify file existence and open it if exists */
        File file = new File(versionFile);
        if (! file.exists()) {
            return "Version file not found";
        }

        try {
            /* Read file */
            FileReader vFile = new FileReader(versionFile);
            BufferedReader br = new BufferedReader(vFile);
            StringBuffer bufOutput = new StringBuffer();

            String line;
            /* Read lines of file */
            while( (line = br.readLine()) != null) {
                bufOutput.append(line);
            }

            /* Close buffer */
            br.close();

            /* Create regular expression object */
            final String REGEX = "^\\d++\\.\\d++\\.\\d++$";
            Pattern pattern = Pattern.compile(REGEX);
            String version = bufOutput.toString();

            if (matchRegExp(pattern, version)) {
                return version;
            } else {
                return "Invalid version";
            }
        } catch (Exception e) {
            /* TODO - return appropriate error message
             * TODO - Log this exception               */
            return null;
        }
    } catch (Exception e) {
        /* TODO - return appropriate error message
         * TODO - Log this exception               */
        return null;
    }
}

/**
 * Return a string after the insertion of <Inep, MacAddr, Timestamp> into
 * database.
 *
 * @author               Danilo Kiyoshi Simizu Yorinori
 * @param     inep       School's INEP number
 * @param     macAddr    Client's Mac Address string
 * @return               String
 */
public static String setAvailability(String inep, String macAddr)
throws Exception {
    /* Insert into database */
    try {
        /* DB data */
        String host = "";
        String port = "";
        String schema = "";
        String user = "";
        String passwd = "";
        String table = "";

        String dateTimestamp = getTimestamp();

        /* Control variable */
        int insert = -1;

        Driver.class.newInstance();

        /* Database connection */
        try {
            String dbConn = createDbConnectionString(host, port, schema);
            Connection conn = DriverManager.getConnection(dbConn, user,
                                                          passwd);

            Statement stmt = conn.createStatement();

            try {
                /* Insert availability information */
                String query = createAvailabilityQuery(table, inep, macAddr,
                                                       dateTimestamp);
                insert = stmt.executeUpdate(query);

                /* Close statement and database connection */
                stmt.close();
                conn.close();

                /**
                 * insert will be greater than 0 if some line was inserted
                 * into database
                 */
                if (insert > 0) {
                    return "Success";
                } else {
                    return "Error";
                }
            } catch (SQLException e) {
                return e.getMessage();
            }
        } catch (SQLException e) {
            return e.getMessage();
        }
    } catch(Exception e) {
        return e.getMessage();
    }
}

/**
 * Return a boolean value after verification of Inep's number existence in
 * database.
 *
 * @author            Danilo Kiyoshi Simizu Yorinori
 * @param     inep    School's INEP number
 * @return            Boolean
 */
public static Boolean verifyId(String inep)
throws Exception {
    try {
        /* DB data */
        String host = "";
        String port = "";
        String schema = "";
        String user = "";
        String passwd = "";
        String table = "";

        Driver.class.newInstance();

        /* Default value */
        Boolean find = false;

        /* Database connection */
        try {
            String dbConn = createDbConnectionString(host, port, schema);
            Connection conn = DriverManager.getConnection(dbConn, user,
                                                          passwd);

            Statement stmt = conn.createStatement();

            try {
                /* Query for inep */
                String query = createIdVerificationQuery(table, inep);
                ResultSet rs = stmt.executeQuery(query);

                /* Verify if query result is not empty */
                if (rs.next() == true) {
                    /* Change return value */
                    find = true;
                }

                /* Close result set, statement and database connection */
                rs.close();
                stmt.close();
                conn.close();

                return find;
            } catch (SQLException e) {
                return false;
            }
        } catch (SQLException e) {
            return false;
        }
    } catch(Exception e) {
        return false;
    }
}

/**
 * Return a string with client/server version.
 *
 * @author               Danilo Kiyoshi Simizu Yorinori
 * @return               String
 */
public static String getAgentVersion()
throws Exception {
    return getVersionFromFile();
}

/**
 * Parse inventory information from XML document, store collected
 * information into database and return a string with insertion result.
 *
 * @author                 Danilo Kiyoshi Simizu Yorinori
 * @param     Inventory    XML string with inventory
 * @return                 String
*/
public static String setInventory(String Inventory) {
    /* Inventory variables */
    String inep = null;
    String memSize = null;
    String kernel = null;
    String type = null;
    String distro = null;
    String model = null;
    String macAddr = null;
    String useUp = null;
    String useDown = null;
    Integer quantity = 0;
    String[] hddSize = new String[0];
    String[] hddModel = new String[0];
    String[] hddUsed = new String[0];
    org.w3c.dom.Document domDoc  = null;

    /* Parse string to xml */
    try {
        /* Create xml string */
        StringReader xmlSt = new StringReader(Inventory);
        /* Transform xml string to readable format for parser */
        InputSource input = new InputSource(xmlSt);
        /* Constructor of builder */
        DOMBuilder builder = new DOMBuilder();
        /* Constructor of parser */
        DOMParser parser = new DOMParser();
        try {
            /* Parser of xml */
            parser.parse(input);
            try {
                /* Create w3c format document */
                domDoc  = parser.getDocument();
            } catch (Exception e) {
                return e.getMessage() + "\nError (2)";
            }
        } catch (Exception e) {
            return e.getMessage() + "\nError (1)";
        }
    } catch (Exception e) {
        return e.getMessage() + "\nError (0)";
    }

    /* Main Tags */
    NodeList collect = domDoc.getElementsByTagName("collect");
    NodeList macTagList = domDoc.getElementsByTagName("mac-address");
    NodeList inepTagList = domDoc.getElementsByTagName("inep");
    NodeList useTagList = domDoc.getElementsByTagName("use");
    NodeList invTagList = domDoc.getElementsByTagName("inventory");
    /* Inventory childs */
    NodeList osTagList = domDoc.getElementsByTagName("os");
    NodeList procTagList = domDoc.getElementsByTagName("processor");
    NodeList memoryTagList = domDoc.getElementsByTagName("memory");
    NodeList diskTagList = domDoc.getElementsByTagName("disk");

    /* Verify root */
    if (collect.getLength() != 1) {
        return "ERROR: wrong number of collect tags";
    }
    /* Verify main tags */
    if (invTagList.getLength() != 1) {
        return "ERROR: wrong number of inventory tags";
    }
    if (macTagList.getLength() != 1) {
        return "ERROR: wrong number of mac-address tags";
    }
    if (inepTagList.getLength() != 1) {
        return "ERROR: wrong number of inep tags";
    }
    if (useTagList.getLength() != 1) {
        return "ERROR: wrong number of use tags";
    }

    /* INEP */
    NodeList inepNumberList = getChildNodes(inepTagList);
    /* INEP Number */
    if (inepNumberList != null) {
        if (inepNumberList.getLength() == 1) {
            /* First child of inep child list */
            Node inepNumberTag = inepNumberList.item(0);
            if (verifyTagName(inepNumberTag, "number") == true) {
                inep = getAttributeValue(inepNumberTag, "value");
                if (inep == null) {
                    return "ERROR: invalid attribute or value of inep number";
                }
            } else {
                return "ERROR: wrong name of tag <number>";
            }
        } else {
            return "ERROR: wrong number of inep number tags";
        }
    } else {
        return "ERROR: inep tag is empty";
    }

    /* MAC */
    NodeList macNumberList = getChildNodes(macTagList);
    /* MAC Number */
    if (macNumberList != null) {
        if (macNumberList.getLength() == 1) {
            /* First child of mac-address child list */
            Node macNumberTag = macNumberList.item(0);
            if (verifyTagName(macNumberTag, "number") == true) {
                macAddr = getAttributeValue(macNumberTag, "value");
                if (macAddr == null) {
                    return "ERROR: invalid attribute or value of mac-address" +
                           " number";
                }
            } else {
                return "ERROR: wrong name of tag <number>";
            }
        } else {
            return "ERROR: wrong number of mac-address number tags";
        }
    } else {
        return "ERROR: mac-address tag is empty";
    }

    /* USE */
    NodeList networkTagList = getChildNodes(useTagList);
    if (networkTagList != null) {
        NodeList networkChilds = getChildNodes(networkTagList);
        if (networkChilds != null) {
            if (networkChilds.getLength() != 2) {
                return "ERROR: wrong number of network childs";
            }
            for (int i = 0; i < 2; i++) {
                Node networkTag = networkChilds.item(i);
                if ((verifyTagName(networkTag, "RX") == true) &&
                    (useDown == null)) {
                    /* RX */
                    useDown = getAttributeValue(networkTag, "value");
                    if (useDown == null) {
                        return "ERROR: invalid attribute or value of RX";
                    }
                } else if ((verifyTagName(networkTag, "TX") == true) &&
                           (useUp == null)) {
                    /* TX */
                    useUp = getAttributeValue(networkTag, "value");
                    if (useUp == null) {
                        return "ERROR: invalid attribute or value of TX";
                    }
                } else {
                    return "ERROR: invalid tag name. Expected: <TX> or <RX>";
                }
            }
        } else {
            return "ERROR: network tag is empty";
        }
    } else {
        return "ERROR: use tag is empty";
    }

    /* INVENTORY */

    /* INVENTORY - PROCESSOR */
    if(procTagList.getLength() == 1) {
        NodeList modelTagList= getChildNodes(procTagList);
        if (modelTagList != null) {
            /* PROCESSOR MODEL */
            if(modelTagList.getLength() == 1) {
                /* First child of processor child list */
                Node modelTag = modelTagList.item(0);
                if (verifyTagName(modelTag, "model") == true) {
                    /* MODEL */
                    model = getAttributeValue(modelTag, "value");
                    if (model == null) {
                        return "ERROR: invalid attribute or value of model";
                    }
                } else {
                    return "ERROR: wrong name of tag <model>";
                }
            } else {
                return "ERROR: wrong number of model tags";
            }
        } else {
            return "ERROR: processor tag is empty";
        }
    } else {
        return "ERROR: wrong number of processor tags";
    }

    /* INVENTORY - OS */
    if(osTagList.getLength() == 1) {
        NodeList osChildList= getChildNodes(osTagList);
        if (osChildList != null) {
            if (osChildList.getLength() != 3) {
                return "ERROR: wrong number of OS child tags";
            }
            /* OS DISTRO */
            NodeList distroTagList = null;
            distroTagList = domDoc.getElementsByTagName("distro");
            if(distroTagList.getLength() == 1) {
                /* First child of distro list */
                Node distroTag = distroTagList.item(0);
                /* DISTRO */
                distro = getAttributeValue(distroTag, "value");
                if (distro == null) {
                    return "ERROR: invalid attribute or value of distro";
                }
            } else {
                return "ERROR: wrong number of distro tags";
            }
            /* OS KERNEL */
            NodeList kernelTagList = null;
            kernelTagList = domDoc.getElementsByTagName("kernel");
            if(kernelTagList.getLength() == 1) {
                /* First child of kernel list */
                Node kernelTag = kernelTagList.item(0);
                /* KERNEL */
                kernel = getAttributeValue(kernelTag, "value");
                if (kernel == null) {
                    return "ERROR: invalid attribute or value of kernel";
                }
            } else {
                return "ERROR: wrong number of kernel tags";
            }
            /* OS TYPE */
            NodeList typeTagList = null;
            typeTagList = domDoc.getElementsByTagName("type");
            if(typeTagList.getLength() == 1) {
                /* First child of type list */
                Node typeTag = typeTagList.item(0);
                /* TYPE */
                type = getAttributeValue(typeTag, "value");
                if (type== null) {
                    return "ERROR: invalid attribute or value of type";
                }
            } else {
                return "ERROR: wrong number of type tags";
            }
        } else {
            return "ERROR: OS tag is empty";
        }
    } else {
        return "ERROR: wrong number of OS tags";
    }

    /* INVENTORY - MEMORY */
    if(memoryTagList.getLength() == 1) {
        NodeList sizeTagList= getChildNodes(memoryTagList);
        /* MEMORY SIZE */
        if (sizeTagList != null) {
            if(sizeTagList.getLength() == 1) {
                /* First child of size child list */
                Node sizeTag = sizeTagList.item(0);
                if (verifyTagName(sizeTag, "size") == true) {
                    /* MODEL */
                    memSize = getAttributeValue(sizeTag, "value");
                    if (memSize == null) {
                        return "ERROR: invalid attribute or value of size";
                    }
                } else {
                    return "ERROR: wrong name of tag <size>";
                }
            } else {
                return "ERROR: wrong number of size tags";
            }
        } else {
            return "ERROR: memory tag is empty";
        }
    } else {
        return "ERROR: wrong number of memory tags";
    }

    /* INVENTORY - DISK */
    if(diskTagList.getLength() == 1) {
        NodeList hddTagList= getChildNodes(diskTagList);

        /* INVENTORY - QUANTITY ATTRIBUTE */
        String hddValue = null;
        if (hddTagList != null) {
            if(hddTagList.getLength() == 1) {
                /* First child of hdd child list */
                Node hddTag = hddTagList.item(0);
                if (verifyTagName(hddTag, "hdd") == true) {
                    /* HDD */
                    hddValue = getAttributeValue(hddTag, "quantity");
                    if (hddValue == null) {
                        return "ERROR: invalid attribute or value of hdd";
                    } else {
                        try {
                            quantity = Integer.parseInt(hddValue);
                        } catch (Exception e) {
                            return "ERROR: impossible to parse quantity " +
                                   "value to int";
                        }
                    }
                } else {
                    return "ERROR: wrong name of tag <hdd>";
                }
            } else {
                return "ERROR: wrong number of hdd tags";
            }
        }

        /* Initialize HDDs size, model and used arrays */
        hddSize = new String[quantity];
        hddModel = new String[quantity];
        hddUsed = new String[quantity];

        /* Auxiliary variables */
        String name = null;
        int length = 0;

        /* HDDs tag list length */
        NodeList hddsTagList = getChildNodes(hddTagList);
        if (hddsTagList == null) {
            length = 0;
        } else {
            length = hddsTagList.getLength();
        }

        /* INVENTORY - DISKS */
        if (quantity == length) {
            for (int i = 0; i < quantity; i++) {
                NodeList hddxTagList = null;
                hddxTagList = domDoc.getElementsByTagName("hdd"+i);
                if (hddxTagList.getLength() != 1) {
                    return "ERROR: hdd" + i + " tag not found";
                }

                NodeList hddiTagList = getChildNodes(hddxTagList);
                if (hddiTagList.getLength() != 3 ) {
                    return "ERROR: wrong number of hdd" + i + " child nodes";
                }
                for (int j = 0; j < 3; j++) {
                    Node hddxChild = hddiTagList.item(j);
                    name = hddxChild.getNodeName();
                    NamedNodeMap hddxChildAttrs = null;
                    hddxChildAttrs = hddxChild.getAttributes();
                    Node hddxChildValue = null;
                    hddxChildValue = hddxChildAttrs.getNamedItem("value");
                    if (hddxChildValue == null) {
                        return "ERROR: " + name + " tag doesn't have " +
                               "attribute \"value\"";
                    }
                    if (name.equals("model")) {
                        hddModel[i] = hddxChildValue.getNodeValue();
                    } else if (name.equals("size")) {
                        hddSize[i] = hddxChildValue.getNodeValue();
                    } else if (name.equals("used")) {
                        hddUsed[i] = hddxChildValue.getNodeValue();
                    } else {
                        return "ERROR: invalid tag name. " +
                               "Expected: <model>, <size> or <used>";
                    }
                }
            }
        } else {
            return "ERROR: wrong number of hddx tags";
        }
    } else {
        return "ERROR: wrong number of disk tags";
    }

    /* End Parse XML */

    try {
        /* Database
         * Generate timestamp */
        String dateTimestamp = getTimestamp();

        /* DB data */
        String host = "";
        String port = "";
        String schema = "";
        String user = "";
        String passwd = "";
        String table = "";

        /* Control variable */
        int insert = -1;

        Driver.class.newInstance();

        /* Database connection */
        try {
            /* Open connection */
            String dbConn = createDbConnectionString(host, port, schema);
            Connection conn = DriverManager.getConnection(dbConn, user,
                                                          passwd);

            Statement stmt = conn.createStatement();

            try {
                String query = null;
                if (quantity == 0) {
                    String quantitySt = quantity.toString();
                    query = createInventoryQuery(table, inep, macAddr, null,
                            null, null, null, null, null, quantitySt, memSize,
                            model, type, distro, kernel, useUp, useDown,
                            dateTimestamp);
                }
                else if (quantity == 1) {
                    query = createInventoryQuery(table, inep, macAddr,
                            hddModel[0], hddSize[0], hddUsed[0], null, null,
                            null, "0", memSize, model, type, distro, kernel,
                            useUp, useDown, dateTimestamp);
                }
                else  {
                    quantity = quantity - 2;
                    String hddAd = quantity.toString();
                    query = createInventoryQuery(table, inep, macAddr,
                            hddModel[0], hddSize[0], hddUsed[0],
                            hddModel[1], hddSize[1], hddUsed[1], hddAd,
                            memSize, model, type, distro, kernel, useUp,
                            useDown, dateTimestamp);
                }
                insert = stmt.executeUpdate(query);
            } catch (SQLException e) {
                return e.getMessage() + "\nError (5)";
            }

            stmt.close();    /* Close statement */
            conn.close();    /* Close database connection */

            if (insert > 0) {
                return "Success";
            } else {
                return "Error";
            }
        } catch (SQLException e) {
            return e.getMessage() + "\nError (4)";
        }
    } catch (Exception e) {
        return e.getMessage() + "\nError (3)";
    }
}

/**
 * Return a string that contains a link to download the newest version of
 * client.
 *
 * @author                    Danilo Kiyoshi Simizu Yorinori
 * @return                    String
 */
public static String getUpdateLink()
{
    /* Strings to package name */
    String baseURL = "";
    String packageName = "";
    String version = null;
    String extension = "";

    /* Create regular expression object */
    final String REGEX = "^\\d++\\.\\d++\\.\\d++$";
    Pattern pattern = Pattern.compile(REGEX);

    /* Get version from file */
    try {
        version = getVersionFromFile();
    } catch (Exception e) {
        return null;
    }

    /* Verify version string */
    if (version == null) {
        return null;
    } else if (version.equals("Invalid version")) {
        return null;
    } else if (version.equals("Version file not found")) {
        return null;
    } else if (matchRegExp(pattern, version)) {
        return baseURL + packageName + version + extension;
    } else {
        return null;
    }
}

public static void main (String[] args) {
    org.w3c.dom.Document domDoc = null;
    String inep = null;
    String Inventory="<?xml version=\"1.0\" encoding=\"UTF-8\"?><collect><inventory><disk type=\"tree\"><hdd quantity=\"2\"><hdd0><model value=\"Maxtor_6E040L0_E16EP1VN\" type=\"string\"/><size value=\"41\" type=\"int\"/><used value=\"28\" type=\"int\"/></hdd0><hdd1><model value=\"Maxtor_6E040L0_E13ME0PN\" type=\"string\"/><size value=\"41\" type=\"int\"/><used value=\"23\" type=\"int\"/></hdd1></hdd></disk><processor><model type=\"string\" value=\"AMD Athlon(TM) XP 1800+\"/></processor><memory><size type=\"int\" value=\"520966144\"/></memory><os><type type=\"string\" value=\"Linux\"/><kernel type=\"string\" value=\"2.6.18\"/><distro type=\"string\" value=\"Debian GNU/Linux squeeze/sid \"/></os></inventory><mac-address><number type=\"string\" value=\"de\"/></mac-address><use><network><RX type=\"int\" value=\"13363934\"/><TX type=\"int\" value=\"13363491\"/></network></use><inep><number type=\"int\" value=\"12345678\"/></inep></collect>";

    /* Parse string to xml */
    try {
        /* Create xml string */
        StringReader xmlSt = new StringReader(Inventory);
        /* Transform xml string to readable format for parser */
        InputSource input = new InputSource(xmlSt);
        /* Constructor of builder */
        DOMBuilder builder = new DOMBuilder();
        /* Constructor of parser */
        DOMParser parser = new DOMParser();
        try {
            /* Parser of xml */
            parser.parse(input);
            try {
                /* Create w3c format document */
                domDoc  = parser.getDocument();
            } catch (Exception e) {
                System.out.println(e.getMessage() + "\nError (2)");
            }
        } catch (Exception e) {
            System.out.println(e.getMessage() + "\nError (1)");
        }
    } catch (Exception e) {
        System.out.println(e.getMessage() + "\nError (0)");
    }

    /* Main Tags */
    NodeList collect = domDoc.getElementsByTagName("collect");
    System.out.println("1");
    NodeList inepTagList = domDoc.getElementsByTagName("inep");
    System.out.println(inepTagList.getLength());
    // =================================== [xx] Testing getChildNodes
    NodeList inepNumberList = getChildNodes(inepTagList);
//                                          ^^^^^^^^^^^
//                                  changeable parameter

    /* INEP Number */
    if (inepNumberList != null) {
        if (inepNumberList.getLength() == 1) {
            /* First child of inep child list */
            Node inepNumberTag = inepNumberList.item(0);
            // ======================================= [xx] verifyTagName
            if (verifyTagName(inepNumberTag, "number") == true) {
            //                               ^^^^^^^^
            //                              changeable parameter
                // ==================================== [xx] getAttributeValue
                inep = getAttributeValue(inepNumberTag, "value");
                //                                      ^^^^^^^
                //                                   changeable parameter
                if (inep == null) {
                    System.out.println("ERROR: invalid attribute or value of inep number");
                } else {
                    System.out.println("INEP number found:" + inep);
                }
            } else {
                System.out.println("ERROR: wrong name of tag <number>");
            }
        } else {
            System.out.println("ERROR: wrong number of inep number tags");
        }
    } else {
        System.out.println("ERROR: inep tag is empty");
    }

    // =================================================== matchRegExp
    /* Create regular expression object */
    final String REGEX = "^\\d++"; // create the regular expression
    Pattern pattern = Pattern.compile(REGEX);
    System.out.println(matchRegExp( Pattern.compile("^\\d++$") , "" ) );

    if (matchRegExp(pattern, inep)) { // test the regular expression
       System.out.println("Inep " + inep + " valid");
    } else {
       System.out.println("Inep " + inep + " invalid");
    }
}

}
