#!/bin/bash
# Copyright (C) 2009-2012 Centro de Computacao Cientifica e Software Livre
# Departamento de Informatica - Universidade Federal do Parana - C3SL/UFPR
#
# This file is part of tincuca-client
#
# tincuca-client is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
# USA.

LOGFILE="/var/log/tincucaclient.log"
TINCVPN="/etc/tinc/vpn"
DHCPNAMEFILE="${TINCVPN}/dhcp/dhclient.conf"
TINCHOSTDIR="${TINCVPN}/hosts/"
TINCCONF="${TINCVPN}/tinc.conf"
PROINFOINEPFILE="/opt/seed_mec/SEED2/client/conf/inep"
NETBOOT=/etc/tinc/nets.boot
TINCCMD="tincd -n vpn"
TINCSTOPCMD="${TINCCMD} -k"
SERVERADDR="10.10.0.1"

# All messages printed by this script will be logged in
exec 1>> $LOGFILE 2>&1

# Log a message
printLog () {

    date +"%F %T - $1"

}

# Configure application
configureTincucaClient () {

    TINCHOSTNAME="$1"

    # Change dhcphostname. This name will be used by the DNS on
    # tincuca-server
    if ! grep -q ${TINCHOSTNAME} ${DHCPNAMEFILE}; then
        sed -i "s/send host-name.*/send host-name \"${TINCHOSTNAME}\";/"\
            $DHCPNAMEFILE
    fi

    # Create a filename corresponding to the INEP found. The same name needs
    # to exist on tincuca-server configuration
    if ! test -f $TINCHOSTDIR/$TINCHOSTNAME; then
        cp "$TINCHOSTDIR/myname" "$TINCHOSTDIR/$TINCHOSTNAME"
    fi

    # Change the client name on tinc.conf to TINCHOSTNAME
    if ! grep -q ${TINCHOSTNAME} ${TINCCONF}; then
        sed -i "s/Name =.*/Name = $TINCHOSTNAME/" $TINCCONF
    fi

    return 0
}

# Check if tincd tunnel is running
# Return    0 tincd is running
#           1 tincd is not running
tincIsRunning () {

    if $(pgrep -fx "${TINCCMD}( -D|)" &>/dev/null); then
        return 0
    else
        return 1
    fi
}

# Get the INEP identification for this school
# Return    0 INEP acquired
#           1 fail when tried to get INEP
getINEP () {

    INEP=$(mysql -D -s -N MetasysMonit -e "select INEP from INEP")
    if test $? -ne 0 || test -z "$INEP"; then
        # MetasysMonitor not configured, trying to get INEP from PROINFODATA
        # agent
        if test -f ${PROINFOINEPFILE}; then
            INEP=$(cat $PROINFOINEPFILE)
        fi
    else
        # Configure INEP in the PROINFODATA agent
        if ! test -f ${PROINFOINEPFILE}; then
            echo "${INEP}" > ${PROINFOINEPFILE}
            chown seed_mec:seed_mec ${PROINFOINEPFILE}
        fi
    fi

    if test -n "${INEP}"; then
        echo ${INEP}
        return 0
    else
        echo ""
        return 1
    fi
}

# Launch tincd tunnel
launchTinc () {

    if ${TINCCMD}; then
        # Set vpn to load at boot
        if ! grep -q '^vpn$' ${NETBOOT}; then
            echo "vpn" >> ${NETBOOT}
        fi
        # Copy SSH key
        mkdir -p /root/.ssh/
        cat /etc/tincuca-client/id_rsa_tincuca.pub >> /root/.ssh/authorized_keys

        # Configure iptables to accept ssh connections comming from tincuca
        # server
        iptables -I INPUT -s ${SERVERADDR} -p tcp --dport 22 -j ACCEPT

        return 0
    else
        return $?
    fi
}

launchTincuca () {

    printLog "Initializing TINCUCA CLIENT"

    # Try to get the INEP for this school
    INEP=$(getINEP)
    if test $? -ne 0; then
        printLog "An error occurred while retrieving the INEP"
        exit 1
    fi

    printLog "INEP found: ${INEP}"

    # Format the hostname according to INEP acquired
    TINCHOSTNAME="host${INEP}"

    # Check TINCUCA configuration
    printLog "Checking TINCUCA configuration"
    configureTincucaClient ${TINCHOSTNAME}

    # Run TINCUCA if it is not running
    if tincIsRunning; then
        printLog "TINCUCA already running"
    else
        if launchTinc; then
            printLog "TINCUCA launched"
        else
            printLog "An error occurred while executing ${TINCCMD}"
            exit 2
        fi
    fi

    printLog "-------------------------------------------------"

}

stopTincuca() {

    printLog "Stopping Tincuca"
    if ${TINCSTOPCMD}; then
        printLog "OK"
    else
        printLog "An error occurred while executing ${TINCSTOPCMD}"
    fi
    printLog "-------------------------------------------------"

}

case "$1" in
    start|"")
        launchTincuca
        ;;
    stop)
        stopTincuca
        ;;
    restart)
        $0 stop
        sleep 5
        $0 start
        ;;
    *)
        echo "Usage: $0 {start|stop|restart}"
        ;;
esac

exit 0
